/**
 * @file audio_file_writer.cpp
 * @brief Audio file writing implementation
 */

#include "media/audio_file_writer.h"

namespace ffvoice {

// Audio file writing is implemented through specialized writers:
// - WAVWriter: Uncompressed PCM audio in WAV format
// - FLACWriter: Lossless compressed audio in FLAC format
// See wav_writer.h and flac_writer.h for implementations

}  // namespace ffvoice
