/**
 * @file audio_file_writer.h
 * @brief Audio file writer interface
 */

#pragma once

#include "ffvoice/types.h"

namespace ffvoice {

// Placeholder - will implement later

}  // namespace ffvoice
