/**
 * @file ring_buffer.cpp
 * @brief Lock-free ring buffer implementation
 */

#include "utils/ring_buffer.h"

namespace ffvoice {

// Template implementation in header file

}  // namespace ffvoice
