/**
 * @file ring_buffer.h
 * @brief Lock-free SPSC ring buffer
 */

#pragma once

#include <array>
#include <atomic>

namespace ffvoice {

// Placeholder - will implement later

}  // namespace ffvoice
